@@ -24,11 +24,11 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
-#include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-href.h>
 #include <string.h>
 #include "yelp-settings.h"
@@ -50,28 +50,33 @@
 #define d(x)
 #endif
 
-static const gchar *color_params[YELP_NUM_COLORS] = {
-    "yelp.color.text",
+static const gchar * const color_params[YELP_NUM_COLORS] = {
+    "yelp.color.fg",
+    "yelp.color.bg",
     "yelp.color.anchor",
-    "yelp.color.background",
     "yelp.color.rule",
-    "yelp.color.base0",
-    "yelp.color.base1",
-    "yelp.color.base2",
-    "yelp.color.base3",
-    "yelp.color.selected0",
-    "yelp.color.selected1",
-    "yelp.color.selected2",
-    "yelp.color.selected3"
+    "yelp.color.gray.fg",
+    "yelp.color.gray.bg",
+    "yelp.color.gray.bg.dark1",
+    "yelp.color.gray.bg.dark2",
+    "yelp.color.gray.bg.dark3",
+    "yelp.color.selected.fg",
+    "yelp.color.selected.bg",
+    "yelp.color.selected.bg.dark1",
+    "yelp.color.selected.bg.dark2",
+    "yelp.color.selected.bg.dark3",
+    "yelp.color.admon.fg",
+    "yelp.color.admon.bg",
+    "yelp.color.admon.bg.dark1",
+    "yelp.color.admon.bg.dark2",
+    "yelp.color.admon.bg.dark3"
 };
 
-static const gchar *icon_params[YELP_NUM_ICONS] = {
+static const gchar * const icon_params[YELP_NUM_ICONS] = {
     "yelp.icon.blockquote",
     "yelp.icon.caution",
     "yelp.icon.important",
-    "yelp.icon.next",
     "yelp.icon.note",
-    "yelp.icon.previous",
     "yelp.icon.programlisting",
     "yelp.icon.tip",
     "yelp.icon.warning"
@@ -140,7 +145,7 @@ yelp_settings_init (void)
 	       If you need an image created, contact the maintainers.
 
 	       Phew, now some notes on which character to use.  Languages that
-	       use guillemets (angle quotations) should use either 00AB or AABB,
+	       use guillemets (angle quotations) should use either 00AB or 00BB,
 	       depending on whether the opening quotation is the left guillemet
 	       or the right guillemet.  Languages that use inverted comma style
 	       quotations should use 201C, 201D, or 201E.  Note that single
@@ -156,15 +161,9 @@ yelp_settings_init (void)
 	case YELP_ICON_IMPORTANT:
 	    icon_names[i] = "yelp-icon-important";
 	    break;
-	case YELP_ICON_NEXT:
-	    icon_names[i] = "stock_next-page";
-	    break;
 	case YELP_ICON_NOTE:
 	    icon_names[i] = "yelp-icon-note";
 	    break;
-	case YELP_ICON_PREVIOUS:
-	    icon_names[i] = "stock_previous-page";
-	    break;
 	case YELP_ICON_PROGRAMLISTING:
 	    icon_names[i] = "yelp-watermark-programlisting";
 	    break;
@@ -285,6 +284,9 @@ yelp_settings_open_preferences (void)
 
 	g_signal_connect (G_OBJECT (prefs_dialog), "response",
 			  G_CALLBACK (gtk_widget_hide), NULL);
+	
+	g_signal_connect (G_OBJECT (prefs_dialog), "delete_event",
+			  G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
 	g_object_unref (glade);
     }
@@ -555,8 +557,8 @@ settings_update (YelpSettingsType type)
     GtkWidget *widget;
     GtkStyle  *style;
     GdkColor  *color;
-    GdkColor   blue = { 0, 0x0000, 0x0000, 0xffff };
-    guint8     max_text;
+    GdkColor   blue = { 0, 0x1E1E, 0x3E3E, 0xE7E7 };
+    guint8     max_text, max_base;
     guint16    rval, gval, bval;
     gint i;
 
@@ -574,50 +576,24 @@ settings_update (YelpSettingsType type)
 	max_text = MAX(style->text[GTK_STATE_NORMAL].red,
 		       MAX(style->text[GTK_STATE_NORMAL].green,
 			   style->text[GTK_STATE_NORMAL].blue   )) >> 8;
+	max_base = MAX(style->base[GTK_STATE_NORMAL].red,
+		       MAX(style->base[GTK_STATE_NORMAL].green,
+			   style->base[GTK_STATE_NORMAL].blue   )) >> 8;
 
-	g_snprintf (colors[YELP_COLOR_TEXT], 8,
+	/* YELP_COLOR_FG */
+	g_snprintf (colors[YELP_COLOR_FG], 8,
 		    "#%02X%02X%02X",
 		    style->text[GTK_STATE_NORMAL].red >> 8,
 		    style->text[GTK_STATE_NORMAL].green >> 8,
 		    style->text[GTK_STATE_NORMAL].blue >> 8);
-	g_snprintf (colors[YELP_COLOR_BACKGROUND], 8,
+	/* YELP_COLOR_BG */
+	g_snprintf (colors[YELP_COLOR_BG], 8,
 		    "#%02X%02X%02X",
 		    style->base[GTK_STATE_NORMAL].red >> 8,
 		    style->base[GTK_STATE_NORMAL].green >> 8,
 		    style->base[GTK_STATE_NORMAL].blue >> 8);
 
-	g_snprintf (colors[YELP_COLOR_RULE], 8,
-		    "#%02X%02X%02X",
-		    ((style->base[GTK_STATE_NORMAL].red >> 8) + 
-		     (style->bg[GTK_STATE_NORMAL].red >> 8) ) / 2,
-		    ((style->base[GTK_STATE_NORMAL].green >> 8) + 
-		     (style->bg[GTK_STATE_NORMAL].green >> 8) ) / 2,
-		    ((style->base[GTK_STATE_NORMAL].blue >> 8) + 
-		     (style->bg[GTK_STATE_NORMAL].blue >> 8) ) / 2);
-
-	for (i = 0; i < 4; i++) {
-	    rval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].red   >> 8) +
-		    i * max_text) / 4;
-	    gval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].green >> 8) +
-		    i * max_text) / 4;
-	    bval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].blue  >> 8) +
-		    i * max_text) / 4;
-	    g_snprintf (colors[YELP_COLOR_BASE0 + i], 8,
-			"#%02X%02X%02X", rval, gval, bval);
-	}
-	for (i = 0; i < 4; i++) {
-	    rval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].red   >> 8) +
-		    i * max_text) / 4;
-	    gval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].green >> 8) +
-		    i * max_text) / 4;
-	    bval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].blue  >> 8) +
-		    i * max_text) / 4;
-	    g_snprintf (colors[YELP_COLOR_SELECTED0 + i], 8,
-			"#%02X%02X%02X", rval, gval, bval);
-	}
-
-	g_object_unref (G_OBJECT (style));
-
+	/* YELP_COLOR_ANCHOR */
 	widget = gnome_href_new ("http://www.gnome.org/", "GNOME");
 	gtk_widget_style_get (widget, "link_color", &color, NULL);
 	if (!color)
@@ -630,6 +606,68 @@ settings_update (YelpSettingsType type)
 	if (color != &blue)
 	    gdk_color_free (color);
 	gtk_object_sink (GTK_OBJECT (widget));
+
+	/* YELP_COLOR_RULE */
+	g_snprintf (colors[YELP_COLOR_RULE], 8,
+		    "#%02X%02X%02X",
+		    ((style->base[GTK_STATE_NORMAL].red >> 8) + 
+		     (style->bg[GTK_STATE_NORMAL].red >> 8) ) / 2,
+		    ((style->base[GTK_STATE_NORMAL].green >> 8) + 
+		     (style->bg[GTK_STATE_NORMAL].green >> 8) ) / 2,
+		    ((style->base[GTK_STATE_NORMAL].blue >> 8) + 
+		     (style->bg[GTK_STATE_NORMAL].blue >> 8) ) / 2);
+
+	/* YELP_COLOR_GRAY_BG */
+	for (i = 0; i < 4; i++) {
+	    rval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].red   >> 8) +
+		    i * max_text) / 4;
+	    gval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].green >> 8) +
+		    i * max_text) / 4;
+	    bval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].blue  >> 8) +
+		    i * max_text) / 4;
+	    g_snprintf (colors[YELP_COLOR_GRAY_BG + i], 8,
+			"#%02X%02X%02X", rval, gval, bval);
+	}
+
+	/* YELP_COLOR_GRAY_FG */
+	g_snprintf (colors[YELP_COLOR_GRAY_FG], 8, "%s",
+		    colors[YELP_COLOR_GRAY_BG_DARK3]);
+
+	/* YELP_COLOR_SELECTED_FG */
+	g_snprintf (colors[YELP_COLOR_SELECTED_FG], 8,
+		    "#%02X%02X%02X",
+		    style->text[GTK_STATE_SELECTED].red >> 8,
+		    style->text[GTK_STATE_SELECTED].green >> 8,
+		    style->text[GTK_STATE_SELECTED].blue >> 8);
+
+	/* YELP_COLOR_SELECTED_BG */
+	for (i = 0; i < 4; i++) {
+	    rval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].red   >> 8) +
+		    i * max_text) / 4;
+	    gval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].green >> 8) +
+		    i * max_text) / 4;
+	    bval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].blue  >> 8) +
+		    i * max_text) / 4;
+	    g_snprintf (colors[YELP_COLOR_SELECTED_BG + i], 8,
+			"#%02X%02X%02X", rval, gval, bval);
+	}
+
+	/* YELP_COLOR_ADMON_FG */
+	g_snprintf (colors[YELP_COLOR_ADMON_FG], 8, "%s",
+		    colors[YELP_COLOR_GRAY_BG_DARK3]);
+
+	/* YELP_COLOR_ADMON_BG */
+	for (i = 0; i < 4; i++) {
+	    gint mult = max_base + ((i * (max_base - max_text)) / 3);
+	    rval = ((255 * mult) / 255);
+	    gval = ((245 * mult) / 255);
+	    bval = ((207 * mult) / 255);
+
+	    g_snprintf (colors[YELP_COLOR_ADMON_BG + i], 8,
+			"#%02X%02X%02X", rval, gval, bval);
+	}
+
+	g_object_unref (G_OBJECT (style));
     }
 
     for (i = 0; i < YELP_SETTINGS_NUM_TYPES; i++)
