@@ -28,6 +28,7 @@
 
 #include <glib/gi18n.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <bonobo/bonobo-main.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -41,12 +42,14 @@
 
 #include "yelp-bookmarks.h"
 #include "yelp-db-pager.h"
+#include "yelp-db-print-pager.h"
 #include "yelp-error.h"
 #include "yelp-html.h"
 #include "yelp-pager.h"
 #include "yelp-settings.h"
 #include "yelp-toc-pager.h"
 #include "yelp-window.h"
+#include "yelp-print.h"
 
 #ifdef ENABLE_MAN
 #include "yelp-man-pager.h"
@@ -54,6 +57,10 @@
 #ifdef ENABLE_INFO
 #include "yelp-info-pager.h"
 #endif
+#ifdef ENABLE_SEARCH
+#include "yelp-search-pager.h"
+#include "gtkentryaction.h"
+#endif
 
 #ifdef YELP_DEBUG
 #define d(x) x
@@ -155,6 +162,8 @@ static void    window_add_widget        (GtkUIManager *ui_manager,
 					 GtkWidget    *vbox);
 static void    window_new_window_cb     (GtkAction *action, YelpWindow *window);
 static void    window_about_document_cb (GtkAction *action, YelpWindow *window);
+static void    window_print_document_cb (GtkAction *action, YelpWindow *window);
+static void    window_print_page_cb    (GtkAction *action, YelpWindow *window);
 static void    window_open_location_cb  (GtkAction *action, YelpWindow *window);
 static void    window_close_window_cb   (GtkAction *action, YelpWindow *window);
 static void    window_copy_cb           (GtkAction *action, YelpWindow *window);
@@ -162,6 +171,7 @@ static void    window_select_all_cb     (GtkAction *action, YelpWindow *window);
 static void    window_find_cb           (GtkAction *action, YelpWindow *window);
 static void    window_preferences_cb    (GtkAction *action, YelpWindow *window);
 static void    window_reload_cb         (GtkAction *action, YelpWindow *window);
+static void    window_enable_cursor_cb  (GtkAction *action, YelpWindow *window);
 static void    window_go_back_cb        (GtkAction *action, YelpWindow *window);
 static void    window_go_forward_cb     (GtkAction *action, YelpWindow *window);
 static void    window_go_home_cb        (GtkAction *action, YelpWindow *window);
@@ -173,6 +183,7 @@ static void    window_about_cb          (GtkAction *action, YelpWindow *window);
 static void    window_copy_link_cb      (GtkAction *action, YelpWindow *window);
 static void    window_open_link_cb      (GtkAction *action, YelpWindow *window);
 static void    window_open_link_new_cb  (GtkAction *action, YelpWindow *window);
+static void    window_copy_mail_cb      (GtkAction *action, YelpWindow *window);
 
 static gboolean           window_load_async      (YelpLoadData *data);
 
@@ -194,12 +205,18 @@ static void               location_response_cb    (GtkDialog       *dialog,
 static gboolean    window_find_action             (YelpWindow        *window);
 static void        window_find_entry_changed_cb   (GtkEditable       *editable,
 						   gpointer           data);
+static void        window_find_entry_activate_cb   (GtkEditable       *editable,
+						    gpointer         data);
+static gboolean    window_find_entry_key_pressed_cb (GtkWidget       *widget,
+						     GdkEventKey     *key,
+						     gpointer         data);
 static void        window_find_save_settings      (YelpWindow        *window);
 static void        window_find_buttons_set_sensitive (YelpWindow      *window,
 						      gboolean        sensitive);
 static void        window_find_clicked_cb         (GtkWidget         *button,
 						   YelpWindow        *window);
-
+static void        window_find_next_cb            (GtkAction *action, 
+						   YelpWindow *window);
 static gboolean    tree_model_iter_following      (GtkTreeModel      *model,
 						   GtkTreeIter       *iter);
 
@@ -234,6 +251,8 @@ struct _YelpWindowPriv {
 
     /* Popup menu*/
     GtkWidget      *popup;
+    gint            merge_id;
+    GtkWidget      *maillink;
     gchar          *uri;
 
     /* Location Information */
@@ -294,6 +313,16 @@ static const GtkActionEntry entries[] = {
       "<Control>N",
       NULL,
       G_CALLBACK (window_new_window_cb) },
+    { "PrintDocument", NULL,
+      N_("Print This Document"),
+      NULL,
+      NULL,
+      G_CALLBACK (window_print_document_cb) },
+    { "PrintPage", NULL,
+      N_("Print This Page"),
+      NULL,
+      NULL,
+      G_CALLBACK (window_print_page_cb) },
     { "AboutDocument", NULL,
       N_("About This Document"),
       NULL,
@@ -326,6 +355,11 @@ static const GtkActionEntry entries[] = {
       "<Control>F",
       NULL,
       G_CALLBACK (window_find_cb) },
+    { "FindNext", NULL,
+      N_("Find Ne_xt"),
+      "<Control>G",
+      NULL,
+      G_CALLBACK (window_find_next_cb) },
     { "Preferences", GTK_STOCK_PREFERENCES,
       N_("_Preferences"),
       NULL, NULL,
@@ -337,6 +371,12 @@ static const GtkActionEntry entries[] = {
       NULL,
       G_CALLBACK (window_reload_cb) },
 
+    { "TextCursor", NULL,
+      "TextCursor",
+      "F7",
+      NULL,
+      G_CALLBACK (window_enable_cursor_cb) },
+
     { "GoBack", GTK_STOCK_GO_BACK,
       N_("_Back"),
       "<Alt>Left",
@@ -399,7 +439,12 @@ static const GtkActionEntry entries[] = {
       N_("_About"),
       NULL,
       NULL,
-      G_CALLBACK (window_about_cb) }
+      G_CALLBACK (window_about_cb) },
+    { "CopyMail", NULL,
+      N_("Copy _Email Address"),
+      NULL,
+      NULL,
+      G_CALLBACK (window_copy_mail_cb) }
 };
 
 GType
@@ -584,6 +629,8 @@ history_step_back (YelpWindow *window)
 	priv->current_doc  = yelp_doc_info_ref (entry->doc_info);
 	priv->current_frag = g_strdup (entry->frag_id);
 	history_entry_free (entry);
+    } else {
+	yelp_window_load (window, "x-yelp-toc:");
     }
 }
 
@@ -595,7 +642,8 @@ history_pop_back (YelpWindow *window)
     GtkAction        *action;
 
     g_return_val_if_fail (YELP_IS_WINDOW (window), NULL);
-    g_return_val_if_fail (window->priv->history_back != NULL, NULL);
+    if (!window->priv->history_back)
+	return NULL;
 
     priv = window->priv;
 
@@ -619,7 +667,8 @@ history_pop_forward (YelpWindow *window)
     GtkAction        *action;
 
     g_return_val_if_fail (YELP_IS_WINDOW (window), NULL);
-    g_return_val_if_fail (window->priv->history_forward != NULL, NULL);
+    if (!window->priv->history_forward)
+	return NULL;
 
     priv = window->priv;
 
@@ -688,6 +737,7 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     YelpWindowPriv *priv;
     YelpDocInfo    *doc_info;
     gchar          *frag_id;
+    GtkAction      *action;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -705,7 +755,7 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 
     priv = window->priv;
 
-    doc_info = yelp_doc_info_get (uri);
+    doc_info = yelp_doc_info_get (uri, FALSE);
     if (!doc_info) {
 	GError *error = NULL;
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_DOC,
@@ -735,6 +785,11 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     if (priv->current_frag)
 	g_free (priv->current_frag);
 
+    action = gtk_action_group_get_action (priv->action_group, 
+						  "PrintDocument");
+    g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
+
+
     priv->current_doc  = yelp_doc_info_ref (doc_info);
     priv->current_frag = g_strdup (frag_id);
 
@@ -766,6 +821,7 @@ window_do_load (YelpWindow  *window,
 		YelpDocInfo *doc_info,
 		gchar       *frag_id)
 {
+    GtkAction      *action;
     YelpWindowPriv *priv;
     GError         *error = NULL;
     gboolean        handled = FALSE;
@@ -778,6 +834,13 @@ window_do_load (YelpWindow  *window,
 
     priv = window->priv;
 
+    action = gtk_action_group_get_action (priv->action_group, 
+					  "PrintDocument");
+    g_object_set (G_OBJECT (action),
+		  "sensitive",
+		  FALSE,
+		  NULL);
+    
     switch (yelp_doc_info_get_type (doc_info)) {
     case YELP_DOC_TYPE_MAN:
 #ifdef ENABLE_MAN
@@ -800,9 +863,22 @@ window_do_load (YelpWindow  *window,
 #endif
 
     case YELP_DOC_TYPE_DOCBOOK_XML:
+	g_object_set (G_OBJECT (action),
+		      "sensitive",
+		      TRUE,
+		      NULL);
     case YELP_DOC_TYPE_TOC:
 	handled = window_do_load_pager (window, doc_info, frag_id);
 	break;
+    case YELP_DOC_TYPE_SEARCH:
+#ifdef ENABLE_SEARCH
+	handled = window_do_load_pager (window, doc_info, frag_id);
+	break;
+#else
+	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
+		     _("Search is not supported in this version."));
+	break;
+#endif
     case YELP_DOC_TYPE_DOCBOOK_SGML:
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
 		     _("SGML documents are no longer supported. Please ask "
@@ -882,6 +958,31 @@ window_error (YelpWindow *window, GError *error, gboolean pop)
     gtk_widget_destroy (dialog);
 }
 
+#ifdef ENABLE_SEARCH
+static char *
+encode_search_uri (const char *search_terms)
+{
+    return g_strdup_printf ("x-yelp-search:%s", search_terms);
+}
+
+static void
+search_activated (GtkAction *action,
+                 YelpWindow *window)
+{
+    const char *search_terms;
+    char *uri;
+
+    search_terms = gtk_entry_action_get_text (GTK_ENTRY_ACTION (action));
+
+    uri = encode_search_uri (search_terms);
+
+    yelp_window_load (window, uri);
+
+    g_free (uri);
+}
+#endif
+
+
 static void
 window_populate (YelpWindow *window)
 {
@@ -903,6 +1004,17 @@ window_populate (YelpWindow *window)
     gtk_action_group_add_actions (priv->action_group,
 				  entries, G_N_ELEMENTS (entries),
 				  window);
+    
+#ifdef ENABLE_SEARCH
+    action =  gtk_entry_action_new ("Search",
+				    _("Search"),
+				    _("Search for other documentation"),
+				    NULL);
+    g_signal_connect (G_OBJECT (action), "activate",
+		      G_CALLBACK (search_activated), window);
+    gtk_action_group_add_action (priv->action_group, action);
+#endif
+
     priv->ui_manager = gtk_ui_manager_new ();
     gtk_ui_manager_insert_action_group (priv->ui_manager, priv->action_group, 0);
 
@@ -919,6 +1031,14 @@ window_populate (YelpWindow *window)
 	window_error (window, error, FALSE);
     }
 
+#ifdef ENABLE_SEARCH
+    if (!gtk_ui_manager_add_ui_from_file (priv->ui_manager,
+					  DATADIR "/yelp/ui/yelp-search-ui.xml",
+					  &error)) {
+	window_error (window, error, FALSE);
+    }
+#endif
+
     yelp_bookmarks_register (window);
 
     gtk_ui_manager_ensure_update (priv->ui_manager);
@@ -929,7 +1049,14 @@ window_populate (YelpWindow *window)
     action = gtk_action_group_get_action (priv->action_group, "GoForward");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-    priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/popup");
+
+    action = gtk_action_group_get_action (priv->action_group, 
+						  "PrintDocument");
+    g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
+
+    priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/main_popup");
+    priv->maillink = gtk_ui_manager_get_widget(priv->ui_manager, "ui/mail_popup");
+    priv->merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
     priv->pane = gtk_hpaned_new ();
     gtk_widget_ref (priv->pane);
@@ -1030,6 +1157,10 @@ window_populate_find (YelpWindow *window, GtkWidget *find_bar)
     priv->find_entry = gtk_entry_new ();
     g_signal_connect (G_OBJECT (priv->find_entry), "changed",
 		      G_CALLBACK (window_find_entry_changed_cb), window);
+    g_signal_connect (G_OBJECT (priv->find_entry), "activate",
+		      G_CALLBACK (window_find_entry_activate_cb), window);
+    g_signal_connect (G_OBJECT (priv->find_entry), "key-press-event",
+		      G_CALLBACK (window_find_entry_key_pressed_cb), window);
     gtk_box_pack_start (GTK_BOX (box), priv->find_entry, TRUE, TRUE, 0);
 
     item = gtk_tool_item_new ();
@@ -1147,6 +1278,15 @@ window_do_load_pager (YelpWindow  *window,
 	case YELP_DOC_TYPE_TOC:
 	    pager = YELP_PAGER (yelp_toc_pager_get ());
 	    break;
+	case YELP_DOC_TYPE_SEARCH:
+#ifdef ENABLE_SEARCH
+	    pager = YELP_PAGER (yelp_search_pager_get (doc_info));
+	    break;
+#else
+	    g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
+			 _("Search is not supported in this version."));
+	    break;
+#endif
 	default:
 	    break;
 	}
@@ -1477,7 +1617,7 @@ window_handle_page (YelpWindow   *window,
     context->length = strlen (page->contents);
 
     uri = yelp_doc_info_get_uri (priv->current_doc,
-				 page->page_id,
+				 priv->current_frag,
 				 YELP_URI_TYPE_FILE);
 
     d (g_print ("  uri            = %s\n", uri));
@@ -1721,6 +1861,20 @@ html_popupmenu_requested_cb (YelpHtml *html,
 {
     YelpWindow *window = YELP_WINDOW (user_data);
 
+    gtk_ui_manager_remove_ui (window->priv->ui_manager,
+			      window->priv->merge_id);
+
+
+    if (g_str_has_prefix(uri, "mailto:")) {
+	gtk_ui_manager_add_ui (window->priv->ui_manager,
+			       window->priv->merge_id,
+			       "/ui/main_popup",
+			       "CpMail",
+			       "CopyMail",
+			       GTK_UI_MANAGER_MENUITEM,
+			       FALSE);
+
+    }
     window->priv->uri = g_strdup (uri);
     gtk_menu_popup (GTK_MENU (window->priv->popup),
 		    NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time());
@@ -1764,7 +1918,7 @@ tree_drag_data_get_cb (GtkWidget         *widget,
     GtkTreeSelection *tree_selection;
     GtkTreeModel     *model;
     GtkTreeIter       iter;
-    gchar *id, *uri;
+    gchar *id, *uri = NULL;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -1791,7 +1945,7 @@ tree_drag_data_get_cb (GtkWidget         *widget,
     case TARGET_URI_LIST:
 	gtk_selection_data_set (selection,
 				selection->target,
-				8, uri, strlen (uri));
+				8, (const guchar *) uri, strlen (uri));
 	break;
     default:
 	g_assert_not_reached ();
@@ -1818,6 +1972,254 @@ window_new_window_cb (GtkAction *action, YelpWindow *window)
     g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0, NULL);
 }
 
+typedef struct {
+    gulong page_handler;
+    gulong error_handler;
+    gulong cancel_handler;
+    gulong finish_handler;
+    YelpPager *pager;
+    YelpWindow *window;
+} PrintStruct;
+
+static void
+print_disconnect (PrintStruct *data)
+{
+    d(g_print ("print disconnect\n"));
+    if (data->page_handler) {
+	g_signal_handler_disconnect (data->pager,
+				     data->page_handler);
+	data->page_handler = 0;
+    }
+    if (data->error_handler) {
+	g_signal_handler_disconnect (data->pager,
+				     data->error_handler);
+	data->error_handler = 0;
+    }
+    if (data->cancel_handler) {
+	g_signal_handler_disconnect (data->pager,
+				     data->cancel_handler);
+	data->cancel_handler = 0;
+    }
+    if (data->finish_handler) {
+	g_signal_handler_disconnect (data->pager,
+				     data->finish_handler);
+	data->finish_handler = 0;
+    }
+    g_free (data);
+}
+
+static void
+print_pager_page_cb (YelpPager *pager,
+		     gchar     *page_id,
+		     gpointer   user_data)
+{
+    PrintStruct *data = user_data;
+    YelpPage    *page;
+    d (g_print ("print_pager_page_cb\n"));
+    d (g_print ("  page_id=\"%s\"\n", page_id));
+
+    page = (YelpPage *) yelp_pager_get_page (pager, page_id);
+
+    if (page) {
+	YelpHtml *html;
+	GtkWidget *gtk_window;
+	int length, offset;
+	char *uri;
+	GtkWidget *vbox = gtk_vbox_new (FALSE, FALSE);
+	d(g_print (page->contents));
+
+	gtk_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	html = yelp_html_new ();
+
+	gtk_container_add (GTK_CONTAINER (gtk_window), GTK_WIDGET (vbox));
+	gtk_box_pack_end (GTK_BOX (vbox), GTK_WIDGET (html), TRUE, TRUE, 0);
+
+	gtk_widget_show (gtk_window);
+	gtk_widget_show (GTK_WIDGET (html));
+	gtk_widget_show (vbox);
+	gtk_widget_hide (gtk_window);
+	uri = yelp_doc_info_get_uri (yelp_pager_get_doc_info (pager),
+				     page_id,
+				     YELP_URI_TYPE_FILE);
+
+	d (g_print ("  uri            = %s\n", uri));
+
+	yelp_html_set_base_uri (html, uri);
+	g_free (uri);
+
+	yelp_html_open_stream (html, "application/xhtml+xml");
+	for (length = strlen (page->contents), offset = 0; length > 0; length -= BUFFER_SIZE, offset += BUFFER_SIZE) {
+	    d(g_print ("data: %.*s\n", MIN (length, BUFFER_SIZE), page->contents + offset));
+	    yelp_html_write (html, page->contents + offset, MIN (length, BUFFER_SIZE));
+	}
+	yelp_html_close (html);
+
+	yelp_print_run (data->window, html, gtk_window, vbox);
+
+	print_disconnect (data);
+
+    }
+}
+
+static void
+print_pager_error_cb (YelpPager   *pager,
+		      gpointer     user_data)
+{
+    PrintStruct *data = user_data;
+    /*     GError *error = yelp_pager_get_error (pager);*/
+
+    d (g_print ("print_pager_error_cb\n"));
+
+    print_disconnect (data);
+}
+
+static void
+print_pager_cancel_cb (YelpPager   *pager,
+		       gpointer     user_data)
+{
+    PrintStruct *data = user_data;
+    d (g_print ("print_pager_cancel_cb\n"));
+
+    print_disconnect (data);
+}
+
+static void
+print_pager_finish_cb (YelpPager   *pager,
+		       gpointer     user_data)
+{
+    PrintStruct *data = user_data;
+
+    d (g_print ("print_pager_finish_cb\n"));
+
+    print_disconnect (data);
+}
+
+static void
+window_print_document_cb (GtkAction *action, YelpWindow *window)
+{
+    PrintStruct *data;
+    YelpPager *pager;
+
+    if (!window->priv->current_doc)
+	return;
+
+    pager = yelp_db_print_pager_new (window->priv->current_doc);
+
+    if (!pager) {
+	return;
+    }
+
+    data = g_new0 (PrintStruct, 1);
+    data->pager = pager;
+    data->window = window;
+
+    data->page_handler =
+	g_signal_connect (data->pager,
+			  "page",
+			  G_CALLBACK (print_pager_page_cb),
+			  data);
+    data->error_handler =
+	g_signal_connect (data->pager,
+			  "error",
+			  G_CALLBACK (print_pager_error_cb),
+			  data);
+    data->cancel_handler =
+	g_signal_connect (data->pager,
+			  "error",
+			  G_CALLBACK (print_pager_cancel_cb),
+			  data);
+    data->finish_handler =
+	g_signal_connect (data->pager,
+			  "finish",
+			  G_CALLBACK (print_pager_finish_cb),
+			  data);
+
+    /* handled = */ yelp_pager_start (data->pager);
+}
+
+static void
+window_print_page_cb (GtkAction *action, YelpWindow *window)
+{
+    GtkWidget *gtk_win;
+    YelpPager  *pager;
+    YelpPage   *page  = NULL;
+    YelpHtml *html;
+    int length, offset;
+    gchar *uri;
+    GtkWidget *vbox = gtk_vbox_new (FALSE, FALSE);
+
+    gtk_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    html = yelp_html_new ();
+    
+    gtk_container_add (GTK_CONTAINER (gtk_win), GTK_WIDGET (vbox));
+    gtk_box_pack_end (GTK_BOX (vbox), GTK_WIDGET (html), TRUE, TRUE, 0);
+    gtk_widget_show (gtk_win);
+    gtk_widget_show (vbox);
+    gtk_widget_show (GTK_WIDGET (html));
+    gtk_widget_hide (gtk_win);
+
+    pager = yelp_doc_info_get_pager (window->priv->current_doc);
+
+    uri = yelp_doc_info_get_uri (window->priv->current_doc, NULL, YELP_URI_TYPE_FILE);
+    
+    yelp_html_set_base_uri (html, uri);
+
+    if (pager) {
+	page = (YelpPage *) yelp_pager_get_page (pager, window->priv->current_frag);
+	
+	yelp_html_open_stream (html, "application/xhtml+xml");
+	for (length = strlen (page->contents), offset = 0; length > 0; length -= BUFFER_SIZE, offset += BUFFER_SIZE) {
+	    d (g_print ("data: %.*s\n", MIN (length, BUFFER_SIZE), page->contents + offset));
+	    yelp_html_write (html, page->contents + offset, MIN (length, BUFFER_SIZE));
+	}
+	yelp_html_close (html);
+    } else { /*html file.  Dump file to window the easy way*/
+	GnomeVFSHandle  *handle;
+	GnomeVFSResult   result;
+	GnomeVFSFileSize n;
+	gchar            buffer[BUFFER_SIZE];	
+
+	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
+	
+	if (result != GNOME_VFS_OK) {
+	    GError *error = NULL;
+	    g_set_error (&error, YELP_ERROR, YELP_ERROR_IO,
+			 _("The file ‘%s’ could not be read.  This file might "
+			   "be missing, or you might not have permissions to "
+			   "read it."),
+			 uri);
+	    window_error (window, error, TRUE);
+	    return;
+	}
+	/* Assuming the file exists.  If it doesn't how did we get this far?
+	 * There are more sinister forces at work...
+	 */
+
+	switch (yelp_doc_info_get_type (window->priv->current_doc)) {
+	case YELP_DOC_TYPE_HTML:
+	    yelp_html_open_stream (html, "text/html");
+	    break;
+	case YELP_DOC_TYPE_XHTML:
+	    yelp_html_open_stream (html, "application/xhtml+xml");
+	    break;
+	default:
+	    g_assert_not_reached ();
+	}
+	
+	while ((result = gnome_vfs_read
+		(handle, buffer, BUFFER_SIZE, &n)) == GNOME_VFS_OK) {
+	    yelp_html_write (html, buffer, n);
+	}
+	
+	yelp_html_close (html);
+
+	
+    
+    }
+    g_free (uri);
+    yelp_print_run (window, html, gtk_win, vbox);
+}
+
 static void
 window_about_document_cb (GtkAction *action, YelpWindow *window)
 {
@@ -1982,6 +2384,21 @@ window_reload_cb (GtkAction *action, YelpWindow *window)
     }
 }
 
+static void
+window_enable_cursor_cb (GtkAction *action, YelpWindow *window)
+{
+    gboolean cursor;
+    GConfClient *gconf_client = gconf_client_get_default ();
+
+    cursor = gconf_client_get_bool (gconf_client,
+				    "/apps/yelp/use_caret",
+				    NULL);
+    gconf_client_set_bool (gconf_client,
+			   "/apps/yelp/use_caret",
+			   !cursor,
+			   NULL);
+}
+
 static gboolean
 window_load_async (YelpLoadData *data)
 {
@@ -2162,6 +2579,25 @@ window_open_link_new_cb (GtkAction *action, YelpWindow *window)
     g_free (window->priv->uri);
 };
 
+static void
+window_copy_mail_cb (GtkAction *action, YelpWindow *window)
+{
+    /*Do things the simple way.
+     * Split the string and remove any query bit then
+     * remove the first 7 chars as they should be mailto:
+     */
+    gchar **split_string = g_strsplit (window->priv->uri, "?", 2);
+    
+    gchar *mail_address = &split_string[0][7];
+    
+    gtk_clipboard_set_text (gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD", 
+								TRUE)),
+			    mail_address,
+			    -1);
+    g_free (window->priv->uri);
+    g_strfreev(split_string);
+}
+
 static void
 window_about_cb (GtkAction *action, YelpWindow *window)
 {
@@ -2255,6 +2691,20 @@ window_find_again (YelpWindow *window, gboolean forward)
     return yelp_html_find_again (priv->html_view, forward);
 }
 
+static void
+window_find_next_cb (GtkAction *action, YelpWindow *window)
+{
+    YelpWindowPriv * priv;
+
+    priv = window->priv;
+    if (!window_find_again (window, TRUE)) {
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+    } else {
+	window_find_buttons_set_sensitive (window, TRUE);
+    }
+}
+
 static void
 window_find_clicked_cb (GtkWidget  *widget,
 			YelpWindow *window)
@@ -2317,7 +2767,54 @@ window_find_entry_changed_cb (GtkEditable *editable,
 }
 
 static void
-window_find_buttons_set_sensitive (YelpWindow *window,
+window_find_entry_activate_cb (GtkEditable *editable,
+				gpointer data)
+{
+    YelpWindow *window;
+    YelpWindowPriv *priv;
+
+    g_return_if_fail (YELP_IS_WINDOW (data));
+
+    window = YELP_WINDOW (data);
+    priv = window->priv;
+
+    window_find_buttons_set_sensitive (window, FALSE);
+
+    /* search forward and update the buttons */
+    if (!window_find_again (window, TRUE)) {
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+    } else {
+	window_find_buttons_set_sensitive (window, TRUE);
+    }
+}
+
+static gboolean
+window_find_entry_key_pressed_cb (GtkWidget    *widget,
+				  GdkEventKey  *key,
+				  gpointer     data)
+{
+    YelpWindow     *window;
+    YelpWindowPriv *priv;
+
+    if (key->keyval == GDK_Escape) {
+	g_return_val_if_fail (YELP_IS_WINDOW (data), FALSE);
+
+	window = YELP_WINDOW (data);
+	priv = window->priv;
+
+	d (g_print ("window_find_entry_key_pressed_cb\n"));
+
+	gtk_widget_hide_all (priv->find_bar);
+
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
+static void
+window_find_buttons_set_sensitive (YelpWindow  *window,
 				   gboolean    sensitive)
 {
     YelpWindowPriv *priv;
